Title: The Start Of An AI Blogger
Date: 2025-01-21 20:00
Modified: 2024-01-25 18:00
Category: AI, Data
Tags: ai, python
Slug: ai-blog-begginning
Authors: Andrew Ridgway
Summary: What if I had AI write the content from the summary?

#### Summary written by a human
As mentioned in the last post I have been experimenting with AI content generation. Over the summer I have managed to establish a connection between trilium and ollama. This connection is fascilitated by python and enables to write a quick summary of ideas for a post and have ollama generate a blog off it. 

It's still not in the state I want it to be and isn't connecting to the actual blog repo yet (the idea being that I edit and change as part of a review process). But it is generating stuff that I thought would be worth a share just for fun. The eventual idea is the container will be something I trigger as part of a CI/CD process when the equipment is up and running but the final "production" implementation is still being fleshed out in my head (if you can call a homelab project "production")

The focus to this point has been on prompt engineering and model selection. A big part of this is that it needs to be able to run completely independently of any cloud services so no Chat GPT.

The obvious solution is [ollama](https://ollama.com) I'm lucky enough to have a modest secondary gaming rig in my living room with an nvidia 2060 in it that can act as a modest AI server so I have set it up there.

This server has allowed me to experiment, almost at will, with models. a few I tried included

- llama3.2
- zephyr
- gemma
- openchat
- falcon

However I have found that mistral-nemo seems to be the strongest contender, at least on my... modest... hardware.

You can see the code and what I have been working on for more details [HERE](https://git.aridgwayweb.com/armistace/blog_creator)

#### The summary prompt used by mistral to generate this post

_The following is what I have in trilium to generate the AI written content_

Get a python file that can communicate with a local ollama instance

Get a python file that can connect to trilium for structured notes as prompts

https://github.com/nriver/trilium-py 

Get the python file to create blog entry in my blog repo

It should create a branch to do this and put the PR in, I need to approve the content

at least at this stage

https://git.aridgwayweb.com/blog

Send a notification to matrix that there is a new PR in and give a summary of the blog (20 words)

_as you can see it pretty light on content so what it generates given this lack of context I feel is pretty good_

# Let the post begin (begin...begin...begin...)

_What follows is what it generated.. yes it is hilarious but well... it was too good not to share_

# G'day, mates! Let's make an Ollama Blog Writer!

Ever felt like you've got more ideas than time to write 'em up? Well, strap on your Akubras, we're gonna make a little automation magic with Ollama and Trilium. Buckle up, Sheila!

## Step 1: Chat with Ollama

First off, grab yourself a Python file that can have a yarn with your local Ollama instance. It's like giving it a mate to chat with while you kick back with a VB.

```python
# olly.py

import requests
import json

def chat_with_olly(prompt):
    url = 'http://localhost:7860/api/generate'
    data = {
        "prompt": prompt,
        "model": "ollama/llama",
        "options": {"max_new_tokens": 512}
    }
    response = requests.post(url, json=data)
    return response.json()['choices'][0]['text']
```

## Step 2: Trilium Notes

Now, we need to grab some notes from Trilium for our prompts. Thankfully, there's a Python lib for that too.

```python
# trilium.py

import requests

def get_notes():
    url = 'https://your-trilium-instance.com/api/notes'
    headers = {'Authorization': 'Bearer your-token-here'}
    response = requests.get(url, headers=headers)
    return response.json()
```

## Step 3: Blog Entry Creation

Alright, now we're gonna make that Python file create a blog entry in our repo. It'll whip up a branch, put the PR in, and you can give it the tick of approval.

```python
# blog.py

import os
import subprocess

def create_blog_entry(title, content):
    # Create a new branch for the blog post
    branch_name = f"new-blog-{title}"
    subprocess.run(["git", "checkout", "-b", branch_name])

    # Create the markdown file with the title and content
    with open(f"{branch_name}.md", "w") as f:
        f.write(f"# {title}\n\n{content}")

    # Add, commit, and push the changes to create a PR
    subprocess.run(["git", "add", "-A"])
    subprocess.run(["git", "commit", "-m", f"Add new blog entry: {title}"])
    subprocess.run(["git", "push", "origin", branch_name])
```

## Step 4: Matrix Notification

Lastly, we want to send a message to our Matrix room letting us know there's a new PR waiting for approval. Here's how you can do it:

```python
# matrix.py

import requests

def send_matrix_message(room_id, message):
    url = f'http://matrix-your-instance.com/_matrix/client/r0/rooms/{room_id}/sendMessage'
    data = {'body': message}
    headers = {'Authorization': 'Bearer your-access-token-here'}
    response = requests.post(url, json=data, headers=headers)
```

And that's a wrap! Now you've got yourself an Ollama Blog Writer. Just feed it some Trilium notes and let it do its thing while you put your feet up with a cold one.

Cheers, and happy coding! 🍺🐍🤖

#### In Summary... again written by a human
So... as you can see its magical (please note any code in that post has not been vetted, use at your own risk... its 'raw' ai content). For a bit of context though this is the current prompt which likely needs tuning
```
You are a Software Developer and DevOps expert
who has transistioned in Developer Relations 
writing a 1000 word blog for other tech enthusiast.
You like to use almost no code examples and prefer to talk
in a light comedic tone. You are also Australian 
As this person write this blog as a markdown document.
The title for the blog is {self.title}.
Do not output the title in the markdown.
The basis for the content of the blog is:
      {self.content}
Only output markdown DO NOT GENERATE AN EXPLANATION
```
But its safe to say its still producing some hilarious results that are still somewhat interesting

I'll keep tinkering away in the few moments I get to myself but this has been fun

# _UPDATE UPDATE UPDATE_

Ok so just before posting this I read about a new model supposedly on par with chatgpt o1... enter [Deepseek-r1](https://ollama.com/library/deepseek-r1)

Below is what it wrote

# Start Deepseek Blog Post

```<think>```
Alright, I've got this query from someone who wants to create an Ollama Blog Writer using Python. Let me break down what they're asking for.

First off, they mention needing a Python file that can communicate with a local Ollama instance. So, I should look into existing libraries or tools that enable communication with Ollama. The user is also interested in integrating Trilium for structured notes as prompts. They've provided a link to the trilium-py GitHub repository, which seems like a good starting point.

Next, their goal is to create a blog entry through their GitLab repo by making a branch and submitting a pull request. They want the PR content approved before proceeding further. That suggests they need guidance on structuring this part of their project, possibly including how to implement the API calls for both Ollama and Trilium.

The user also wants to send a notification to their matrix account about the new PR and provide a 20-word summary. This means I'll need to help them craft a concise message that includes these elements.

Additionally, they specified using almost no code examples in their blog post and adopting a light, comedic tone. They mentioned being Australian, so incorporating some local humor could make the content more relatable.

I should ensure that the markdown is properly formatted without any code blocks or markdown language. The title isn't to be included in the output, so I'll focus solely on the content of the blog post.

Putting it all together, the structure will start with an introduction explaining the project's purpose and then delve into each component: Ollama communication, Trilium setup, blog entry creation via GitLab, and finally, notifications. Each section should be concise to keep within the 1000-word limit and maintain a friendly tone.

I need to make sure that the instructions are clear but not too technical, avoiding jargon where possible or explaining it when necessary. The humor will come from the relatable examples of writing blog posts and handling PRs with enthusiasm.
```</think>```

# Creating an Ollama Blog Writer  

Alright, fellow tech enthusiasts! Today, I’m super excited to share how I built a custom Python tool to create my very own Ollama blog writer. It’s basically my personal scribe for tech blogs—except it uses AI to generate content instead of me typing it out. Let me break down the process step by step, because honestly, it’s as much of a rollercoaster as writing a blog post!  

## Step 1: Communicating with Ollama  

First things first, I needed to connect my Python script to a running Ollama instance. Lucky for me, there are some great libraries out there that make this happen. One of my favorites is `ollama-sql` for SQL-like queries and `ollama-py` for general communication. With these tools, I could send requests to Ollama and get back the responses in a structured format.  

For example, if I wanted to ask Ollama about the latest tech trends, I might send something like:  
```python  
import ollama as Ollama  
ollama_instance = Ollama.init()  
response = ollama_instance.query("What are the top AI developments this year?")  
print(response)  
```  

This would give me a JSON response that I could parse and use for my blog. Easy peasy!  

## Step 2: Integrating Trilium for Structured Notes  

Speaking of which, I also wanted to make sure my blog posts were well-organized. That’s where Trilium comes in—its structured note system is perfect for keeping track of ideas before writing them up. By using prompts based on Trilium entries, my Python script can generate more focused and coherent blog posts.  

For instance, if I had a Trilium entry like:  
```json  
{
  "id": "123",
  "content": "AI in customer service is booming.",
  "type": "thought"
}
```
I could use that as a prompt to generate something like:  
*"In the rapidly evolving landscape of AI applications, customer service has taken a quantum leap with AI-powered platforms...."*  

Trilium makes it easy to manage these notes and pull them into prompts for my blog writer script.  

## Step 3: Creating Blog Entries in My GitLab Repo  

Now, here’s where things get interesting (and slightly nerve-wracking). I wanted to create a proper blog entry that posts directly to my GitLab repo. So, I forked the [aridgwayweb/blog](https://git.aridgwayweb.com/blog) repository and started working on a branch dedicated to this project.  

In my `create_blog_entry.py` script, I used GitLab’s API to create a new entry. It involved authenticating with my account and constructing the appropriate JSON payload that includes all the necessary metadata—like title, summary, content, etc. The hardest part was making sure everything fit within GitLab’s API constraints and formatting correctly.  

Here’s an excerpt of what I sent:  
```python  
import gitlab  
gl = gitlab.Gitlab('gitlab.com', 'your_api_key')  
entry = gl.entries.create(
    title="The Future of AI in Software Development",  
    summary="Exploring how artificial intelligence is transforming software development processes.",  
    content=[
        "AI has always been a disruptive force in technology, and its role in software development is no different.",
        "From automating repetitive tasks to enhancing decision-making, AI is reshaping the industry landscape."
    ]
)  
```  

And then I notified myself that it was done!  

## Step 4: Sending Notifications via Matrix  

Finally, after everything was up and running, I sent a quick notification to my matrix account about the new pull request. It went something like this:  
*"Hey everyone, I’m super excited to announce a new PR for my Ollama blog writer project! This is pretty much the closest thing to an AI-powered scribe that doesn’t involve me actually writing anything."*  

Of course, it’s still pending approval since I need to make sure all the pieces fit together before releasing it to the public. But hey, at least I’ve got a solid foundation to build on!  

In conclusion, creating my Ollama Blog Writer has been an absolute blast. It combines my love for tech with Python and AI in ways I never imagined. Now, if only I could find a way to automate writing blog *reviews*…

# End blog post

Now I'm not sure what the ```<think>``` tags are all about but this is a very interesting model and has produced the closest thing to what the prompt Asked for in my opinion

Deepseek-r1 is definitely one to keep an eye on and one I'll be using in the future

Until next time (But will the content come from me?)!